import ParseError from '../ParseError.js';
import { createOptionsForPrimitive, createExact } from '../util.js';
const inRange = (value, min, max) => {
    const parsedValue = value;
    return parsedValue >= min && parsedValue <= max;
};
const isNumber = (type) => 'number' === type;
const orElse = (u, value, parseAble, min, max) => (parseAble && inRange(value, min, max) ? value : u);
const orElseThrow = (error, value, parseAble, min, max) => {
    if (parseAble && inRange(value, min, max)) {
        return value;
    }
    throw error;
};
const parseAsNumber = (value) => {
    const actualType = typeof value;
    const expectedType = 'number';
    return {
        ...createOptionsForPrimitive(value, expectedType, actualType),
        inRangeOf: (min, max) => {
            if (min > max || max < min) {
                throw new Error(`min must be less than max, max must also be greater than min. Min: ${min}, Max: ${max}`);
            }
            const options = {
                orElseGet: (u) => orElse(u, value, isNumber(actualType), min, max),
                orElseLazyGet: (callBackFunction) => isNumber(actualType) && inRange(value, min, max)
                    ? value
                    : callBackFunction(),
                orElseGetUndefined: () => orElse(undefined, value, isNumber(actualType), min, max),
                orElseGetNull: () => orElse(null, value, isNumber(actualType), min, max),
                orElseThrowDefault: (name) => orElseThrow(ParseError.defaultMessage(name, value, expectedType, actualType), value, isNumber(actualType), min, max),
                orElseThrowCustom: (message) => orElseThrow(ParseError.customizedMessage(message), value, isNumber(actualType), min, max),
            };
            return options;
        },
        exactlyAs: (t) => ({
            ...createExact(value, expectedType, typeof value, t),
        }),
    };
};
export default parseAsNumber;
//# sourceMappingURL=parseAsNumber.js.map