export declare const isEqual: (c: string, d: string) => boolean;
export declare type Options<T> = Readonly<{
    /**
     * @param u the alternative value to return if the data type does not match and have failed to parse, precompute instead of lazily loaded
     * @returns the parsed value, otherwise the alternative value
     */
    orElseGet: <U>(u: U) => U | T;
    /**
     * @param callBackFunction that is lazily loaded and will be called when it's necessary
     * @returns the parsed value, otherwise the alternative value
     */
    orElseLazyGet: <U>(callBackFunction: () => U) => U | T;
    /**
     * @returns the parsed value, otherwise the undefined
     * */
    orElseGetUndefined: () => undefined | T;
    /**
     * @returns the parsed value, otherwise the null
     * */
    orElseGetNull: () => null | T;
    /**
     * @name the name of the variable to be shown when Error is thrown
     *
     * ```ts
     * Example:
     * const name = 'example'
     * Expect "name": "example" to have type of "int", actual type is "string"
     * ```
     *
     * @returns the parsed value
     * */
    orElseThrowDefault: (name: string) => T;
    /**
     * @message the message to be shown when Error is thrown
     * @returns the parsed value
     * */
    orElseThrowCustom: (message: string) => T;
}>;
export declare const isExact: <T, M extends T>(m: M, u: unknown, parseAble: boolean) => boolean;
export declare const createExact: <T, M extends T>(value: unknown, expectedType: string, actualType: string, m: M) => Readonly<{
    /**
     * @param u the alternative value to return if the data type does not match and have failed to parse, precompute instead of lazily loaded
     * @returns the parsed value, otherwise the alternative value
     */
    orElseGet: <U>(u: U) => T | U;
    /**
     * @param callBackFunction that is lazily loaded and will be called when it's necessary
     * @returns the parsed value, otherwise the alternative value
     */
    orElseLazyGet: <U_1>(callBackFunction: () => U_1) => T | U_1;
    /**
     * @returns the parsed value, otherwise the undefined
     * */
    orElseGetUndefined: () => T | undefined;
    /**
     * @returns the parsed value, otherwise the null
     * */
    orElseGetNull: () => T | null;
    /**
     * @name the name of the variable to be shown when Error is thrown
     *
     * ```ts
     * Example:
     * const name = 'example'
     * Expect "name": "example" to have type of "int", actual type is "string"
     * ```
     *
     * @returns the parsed value
     * */
    orElseThrowDefault: (name: string) => T;
    /**
     * @message the message to be shown when Error is thrown
     * @returns the parsed value
     * */
    orElseThrowCustom: (message: string) => T;
}>;
export declare const createOptionsForPrimitive: <T>(value: unknown, expectedType: string, actualType: string) => Readonly<{
    /**
     * @param u the alternative value to return if the data type does not match and have failed to parse, precompute instead of lazily loaded
     * @returns the parsed value, otherwise the alternative value
     */
    orElseGet: <U>(u: U) => T | U;
    /**
     * @param callBackFunction that is lazily loaded and will be called when it's necessary
     * @returns the parsed value, otherwise the alternative value
     */
    orElseLazyGet: <U_1>(callBackFunction: () => U_1) => T | U_1;
    /**
     * @returns the parsed value, otherwise the undefined
     * */
    orElseGetUndefined: () => T | undefined;
    /**
     * @returns the parsed value, otherwise the null
     * */
    orElseGetNull: () => T | null;
    /**
     * @name the name of the variable to be shown when Error is thrown
     *
     * ```ts
     * Example:
     * const name = 'example'
     * Expect "name": "example" to have type of "int", actual type is "string"
     * ```
     *
     * @returns the parsed value
     * */
    orElseThrowDefault: (name: string) => T;
    /**
     * @message the message to be shown when Error is thrown
     * @returns the parsed value
     * */
    orElseThrowCustom: (message: string) => T;
}>;
declare type MutableDataStructType = 'Map' | 'Array' | 'Set';
export declare const structTypeOfEqual: (value: unknown, immutable: boolean, type: MutableDataStructType) => boolean;
export declare const freeze: <G>(g: G, freeze: boolean) => G;
export declare const createOptionsForStructure: <T extends object>(value: unknown, callBackFunction: () => T, immutable: boolean, type: MutableDataStructType) => Readonly<{
    /**
     * @param u the alternative value to return if the data type does not match and have failed to parse, precompute instead of lazily loaded
     * @returns the parsed value, otherwise the alternative value
     */
    orElseGet: <U>(u: U) => T | U;
    /**
     * @param callBackFunction that is lazily loaded and will be called when it's necessary
     * @returns the parsed value, otherwise the alternative value
     */
    orElseLazyGet: <U_1>(callBackFunction: () => U_1) => T | U_1;
    /**
     * @returns the parsed value, otherwise the undefined
     * */
    orElseGetUndefined: () => T | undefined;
    /**
     * @returns the parsed value, otherwise the null
     * */
    orElseGetNull: () => T | null;
    /**
     * @name the name of the variable to be shown when Error is thrown
     *
     * ```ts
     * Example:
     * const name = 'example'
     * Expect "name": "example" to have type of "int", actual type is "string"
     * ```
     *
     * @returns the parsed value
     * */
    orElseThrowDefault: (name: string) => T;
    /**
     * @message the message to be shown when Error is thrown
     * @returns the parsed value
     * */
    orElseThrowCustom: (message: string) => T;
}>;
export {};
//# sourceMappingURL=util.d.ts.map