import ParseError from './ParseError.js';
export const isEqual = (c, d) => c === d;
const orElseThrow = (t, parseAble, error) => {
    if (parseAble) {
        return t;
    }
    throw error;
};
export const isExact = (m, u, parseAble) => parseAble && u === m;
const createOptionsWithParseable = (value, expectedType, actualType, isParseable) => ({
    orElseGet: (u) => (isParseable ? value : u),
    orElseLazyGet: (callBackFunction) => isParseable ? value : callBackFunction(),
    orElseGetUndefined: () => (isParseable ? value : undefined),
    orElseGetNull: () => (isParseable ? value : null),
    orElseThrowDefault: (name) => orElseThrow(value, isParseable, ParseError.defaultMessage(name, value, expectedType, actualType)),
    orElseThrowCustom: (message) => orElseThrow(value, isParseable, ParseError.customizedMessage(message)),
});
export const createExact = (value, expectedType, actualType, m) => createOptionsWithParseable(value, `${m}`, actualType, isExact(m, value, isEqual(expectedType, actualType)));
export const createOptionsForPrimitive = (value, expectedType, actualType) => createOptionsWithParseable(value, expectedType, actualType, isEqual(expectedType, actualType));
const getMutableType = (variableValue) => Object.prototype.toString.call(variableValue).slice(8, -1);
const getIterableExpectedAndactualType = (value, immutable, type) => {
    const mutableType = getMutableType(value);
    return {
        expectedType: immutable ? `Readonly${type}` : type,
        actualType: immutable ? `Readonly${mutableType}` : mutableType,
    };
};
export const structTypeOfEqual = (value, immutable, type) => {
    const { expectedType, actualType } = getIterableExpectedAndactualType(value, immutable, type);
    return isEqual(expectedType, actualType);
};
export const freeze = (g, freeze) => freeze && typeof g === 'object' ? Object.freeze(g) : g;
export const createOptionsForStructure = (value, callBackFunction, immutable, type) => ({
    orElseGet: (u) => freeze(structTypeOfEqual(value, immutable, type) ? callBackFunction() : u, immutable),
    orElseLazyGet: (lazyCallBackFunction) => freeze(structTypeOfEqual(value, immutable, type)
        ? callBackFunction()
        : lazyCallBackFunction(), immutable),
    orElseGetUndefined: () => structTypeOfEqual(value, immutable, type)
        ? freeze(callBackFunction(), immutable)
        : undefined,
    orElseGetNull: () => structTypeOfEqual(value, immutable, type)
        ? freeze(callBackFunction(), immutable)
        : null,
    orElseThrowDefault: (name) => {
        if (structTypeOfEqual(value, immutable, type)) {
            return freeze(callBackFunction(), immutable);
        }
        const { expectedType, actualType } = getIterableExpectedAndactualType(value, immutable, type);
        throw ParseError.defaultMessage(name, value, expectedType, actualType);
    },
    orElseThrowCustom: (message) => {
        if (structTypeOfEqual(value, immutable, type)) {
            return freeze(callBackFunction(), immutable);
        }
        throw ParseError.customizedMessage(message);
    },
});
//# sourceMappingURL=util.js.map